package net.wanji.opt.controllerv2;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.servicev2.TrendServiceV2;
import net.wanji.opt.servicev2.TrunkLineService;
import net.wanji.opt.servicev2.TrunkLineStatusDisService;
import net.wanji.opt.synthesis.pojo.TrunkLineProblemDescribeEntity;
import net.wanji.opt.vo2.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhengyifan
 * @date 2025/3/14
 */
@Api(value = "TrunkLineController", description = "监测详情-干线")
@RequestMapping("/trunkLine")
@RestController
@Slf4j
public class TrunkLineController {
    @Resource
    private TrunkLineStatusDisService trunkLineStatusDisService;
    @Resource
    private TrunkLineService trunkLineService;

    @GetMapping("/statusDisInfoList")
    @ApiOperation(httpMethod="GET",value="干线数据", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenID", value = "干线ID", required = true, dataType = "String",defaultValue = ""),
            @ApiImplicitParam(name = "date", value = "日期", required = true, dataType = "String",defaultValue = ""),
            @ApiImplicitParam(name = "timeGranule", value = "时间颗粒度 支持参数：5 15 30 60", required = true, dataType = "String",defaultValue = "5"),

    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventStatisticPo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject statusDisInfoList(String greenID, String date, String timeGranule) {
        try {
            JsonViewObject object = JsonViewObject.newInstance();
            JSONObject list = trunkLineStatusDisService.trunkLineStatusDisInfoList(greenID, date, timeGranule);
            return object.success(list);
        } catch (Exception e) {
            log.error("监测详情-干线事件详情-干线交通状态分布趋势: ", e);
            return JsonViewObject.newInstance().fail("请求失败");
        }
    }

    @GetMapping("/problemDescribe")
    @ApiOperation(httpMethod="GET",value="干线问题描述", notes="干线问题描述", response = JsonViewObject.class)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenID", value = "干线ID", required = true, dataType = "String",defaultValue = ""),
            @ApiImplicitParam(name = "time", value = "日期", required = false, dataType = "String",defaultValue = ""),

    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = TrunkLineProblemDescribeVO.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject problemDescribe(String greenID , String time ) {
        try {
            JsonViewObject object = JsonViewObject.newInstance();
            List<TrunkLineProblemDescribeEntity> list = trunkLineService.getTrunkLineProblemDescribe(greenID,time);
            return object.success(list);
        } catch (Exception e) {
            log.error("监测详情-干线详情-干线问题描述: ", e);
            return JsonViewObject.newInstance().fail("干线问题描述查询失败");
        }
    }

    @ApiOperation(value = "干线实时告警", notes = "干线实时告警", response = JsonViewObject.class, httpMethod="GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenID", value = "绿波ID", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "time", value = "时间 格式：yyyy-MM-dd HH:ii:ss", dataType = "String", paramType = "query"),
    })
    @GetMapping(value = "/getTrunkLineRealTimeAlarm")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossRealTimeAlarmVO.class),
    })
    public JsonViewObject getTrunkLineRealTimeAlarm(Integer greenID, @RequestParam(defaultValue = "") String time) {
        try {
            JsonViewObject object = JsonViewObject.newInstance();
            List<CrossRealTimeAlarmVO> list = trunkLineService.getTrunkLineRealTimeAlarm(greenID,time);
            return object.success(list);
        } catch (Exception e) {
            log.error("获取干线实时告警失败: ", e);
            return JsonViewObject.newInstance().fail("获取干线实时告警失败");
        }
    }

    @ApiOperation(value = "干线路口问题", notes = "干线路口问题", response = JsonViewObject.class, httpMethod="GET")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "greenID", value = "绿波ID", required = true, dataType = "Integer", paramType = "query"),
            @ApiImplicitParam(name = "time", value = "时间 格式：yyyy-MM-dd HH:ii:ss", dataType = "String", paramType = "query"),
    })
    @GetMapping(value = "/getTrunkLineCrossProblem")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = TrunkLineCrossProblemVO.class),
    })
    public JsonViewObject getTrunkLineCrossProblem(Integer greenID, @RequestParam(defaultValue = "") String time) {
        try {
            JsonViewObject object = JsonViewObject.newInstance();
            List<TrunkLineCrossProblemVO> list = trunkLineService.getTrunkLineCrossProblem(greenID,time);
            return object.success(list);
        } catch (Exception e) {
            log.error("获取干线路口问题失败: ", e);
            return JsonViewObject.newInstance().fail("获取干线路口问题失败");
        }
    }
}
