package net.wanji.opt.servicev2.implv2;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveCrossMapper;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.common.enums.EventInfoTypeEnum;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dao.mapper.GreenWaveRealTimeMapperV2Mapper;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.servicev2.TrendServiceV2;
import net.wanji.opt.synthesis.pojo.StrategyCrossResultEntity;
import net.wanji.opt.vo2.*;
import net.wanji.opt.vo2.dto.CrossLastOptResultDTO;
import net.wanji.opt.vo2.dto.GreenLastOptResultDTO;
import net.wanji.opt.vo2.dto.GreenOptDTO;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/03/06 18:44
 */
@Service
@Slf4j
public class TrendServiceV2Impl implements TrendServiceV2 {

    @Resource
    private HoloEventMapper holoEventMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private GreenWaveRealTimeMapperV2Mapper greenWaveRealTimeMapperV2Mapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;
    @Resource
    private StrategyGreenOptHistMapper greenOptHistMapper;
    @Resource
    private GreenwaveCrossMapper greenwaveCrossMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private RedisUtils redisUtil;
    @Resource
    private BaseCrossSchemeMapper baseCrossSchemeMapper;
    @Resource
    private BaseCrossInfoCache baseCrossInfoCache;

    private static List<OptMonitoringVO> greenListCache = new ArrayList<>(10);
    private static List<OptMonitoringVO> crossListCache = new ArrayList<>(80);


    @Override
    public List<CrossGreenStatusTimeRateVO> crossGreenStatusTimeRate() {
        List<CrossGreenStatusTimeRateVO> crossGreenStatusTimeRateVOS = holoEventMapper.selectCrossGreenStatusTimeRate();
        if (!CollectionUtils.isEmpty(crossGreenStatusTimeRateVOS)) {
            Map<String, CrossGreenStatusTimeRateVO> map = crossGreenStatusTimeRateVOS.stream().collect(Collectors.toMap(CrossGreenStatusTimeRateVO::getType, crossGreenStatusTimeRateVO -> crossGreenStatusTimeRateVO));
            for (EventInfoTypeEnum typeEnum : EventInfoTypeEnum.values()) {
                String eventType = typeEnum.getEventType();
                if (!map.containsKey(eventType)) {
                    CrossGreenStatusTimeRateVO vo = new CrossGreenStatusTimeRateVO();
                    vo.setType(eventType);
                    vo.setCount(0);
                    crossGreenStatusTimeRateVOS.add(vo);
                }
            }

        }
        return crossGreenStatusTimeRateVOS;
    }

    @Override
    public List<CrossOptInfoVO> crossOptInfoList() throws Exception {
        List<CrossOptInfoVO> results = new ArrayList<>();
        try {
            results = new ArrayList<>();
            List<CrossOptInfoVO> resultList = strategyCrossResultMapper.selectAICrossListV2();
            long start = DateUtil.getCurrentTimeSecond();
            long end = start + 5 * 60;
            LambdaQueryWrapper<CrossDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.between(CrossDataHistPO::getBatchTime, start, end);
            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(resultList)) {
                for (CrossOptInfoVO entity : resultList) {
                    CrossOptInfoVO result = new CrossOptInfoVO();
                    String crossId = entity.getCrossId();
                    Double trafficIndex = entity.getTrafficIndex();
                    Integer queueLength = entity.getQueueLength();
                    String location = entity.getLocation();
                    location = location.replace("POINT(", "").replace(" ", ",").replace(")", "");
                    BeanUtils.copyProperties(entity, result);
                    result.setLocation(location);
                    for (CrossDataHistPO crossDataHistPO : crossDataHistPOS) {
                        String lastWeekCrossId = crossDataHistPO.getCrossId();
                        Double lastWeekTrafficIndex = crossDataHistPO.getTrafficIndex();
                        Double lastWeekQueueLength = crossDataHistPO.getQueueLength();
                        if (StringUtils.endsWithIgnoreCase(crossId, lastWeekCrossId)) {
                            if (trafficIndex > lastWeekTrafficIndex) {
                                result.setTrafficIndexUpDown(1);
                            } else {
                                result.setTrafficIndexUpDown(0);
                            }
                            if (queueLength > lastWeekQueueLength) {
                                result.setQueueLengthUpDown(1);
                            } else {
                                result.setQueueLengthUpDown(0);
                            }
                            results.add(result);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("态势监测-区域体检-雷达图-查询失败:{}", e);
            throw new Exception(e);
        }
        return results;
    }

    @Override
    public List<GreenOptInfoVO> greenOptInfoList() throws Exception {
        List<GreenOptInfoVO> results = new ArrayList<>();
        List<GreenOptDTO> greenOptDTOS = greenWaveRealTimeMapperV2Mapper.selectGreenOptInfoList();
        long start = DateUtil.getCurrentTimeSecond();
        long end = start + 5 * 60;
        LambdaQueryWrapper<GreenwaveHistPO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.between(GreenwaveHistPO::getBatchTime, start, end);
        List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(greenOptDTOS)) {
            for (GreenOptDTO greenOptDTO : greenOptDTOS) {
                Integer greenId = greenOptDTO.getGreenId();
                Double trafficIndex = greenOptDTO.getTrafficIndex();
                Integer speed = greenOptDTO.getSpeed();
                Integer travelTime = greenOptDTO.getTravelTime();
                greenOptDTO.setStrategyName("效率提升");
                String roadDirection = greenOptDTO.getRoadDirection();
                greenOptDTO.setDirectionName(GreenBeltDirEnum.getDesc(roadDirection));
                for (GreenwaveHistPO greenwaveHistPO : greenwaveHistPOS) {
                    Integer curGreenId = greenwaveHistPO.getGreenId();
                    String curDirection = greenwaveHistPO.getRoadDirection();
                    Double curTrafficIndex = greenwaveHistPO.getTrafficIndex();
                    Double curSpeed = Objects.nonNull(greenwaveHistPO.getSpeed()) ? greenwaveHistPO.getSpeed() : 0;
                    Integer curTravelTime = greenwaveHistPO.getTrvalTime();
                    if (Objects.equals(greenId, curGreenId) && StringUtils.endsWithIgnoreCase(roadDirection, curDirection)) {

                        if (trafficIndex > curTrafficIndex) {
                            greenOptDTO.setTrafficIndexUpDown(1);
                        } else {
                            greenOptDTO.setTrafficIndexUpDown(0);
                        }
                        if (speed > curSpeed.intValue()) {
                            greenOptDTO.setSpeedUpDown(1);
                        } else {
                            greenOptDTO.setSpeedUpDown(0);
                        }
                        if (travelTime > curTravelTime) {
                            greenOptDTO.setTravelUpDown(1);
                        } else {
                            greenOptDTO.setTravelUpDown(0);
                        }
                    }

                }
            }
            Map<Integer, List<GreenOptDTO>> greenIdMap = greenOptDTOS.stream().collect(Collectors.groupingBy(GreenOptDTO::getGreenId));
            for (Map.Entry<Integer, List<GreenOptDTO>> entry : greenIdMap.entrySet()) {
                GreenOptInfoVO greenOptInfoVO = new GreenOptInfoVO();
                Integer greenId = entry.getKey();
                List<GreenOptDTO> value = entry.getValue();
                GreenOptDTO greenOptDTO = value.get(0);
                greenOptInfoVO.setGreenId(greenId);
                greenOptInfoVO.setGreenName(greenOptDTO.getGreenName());
                greenOptInfoVO.setDetails(value);
                results.add(greenOptInfoVO);
            }
        }
        return results;
    }

    @Override
    public List<OptMonitoringVO> greenOptMonitoringList() throws Exception {
        List<OptMonitoringVO> optMonitoringVOS = holoEventMapper.selectGreenOptMonitoringList();
        List<GreenLastOptResultDTO> resultDTOS = greenOptHistMapper.selectLastGreenOptResultList();
        long current = System.currentTimeMillis();
        optMonitoringVOS.forEach(optMonitoringVO -> {
            String id = optMonitoringVO.getId();
            optMonitoringVO.setTypeDesc(EventInfoTypeEnum.getOptDesc(optMonitoringVO.getType()));
            if (!CollectionUtils.isEmpty(resultDTOS)) {
                for (GreenLastOptResultDTO resultDTO : resultDTOS) {
                    Integer greenId = resultDTO.getGreenId();
                    if (Objects.equals(greenId, Integer.valueOf(id))) {
                        Integer controlMethod = resultDTO.getControlMethod();
                        Date optTime = resultDTO.getOptTime();
                        Integer duration = resultDTO.getDuration();
                        if (Objects.equals(-1, controlMethod)) {
                            optMonitoringVO.setOptStatus(0);
                            //optMonitoringVO.setOptStatusDesc("");
                        } else if (current < optTime.getTime() + duration) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                        }
                    }
                }
            }
        });
        greenListCache = optMonitoringVOS;
        return optMonitoringVOS;
    }

    @Override
    public List<OptMonitoringVO> crossOptMonitoringList() throws Exception {
        List<OptMonitoringVO> optMonitoringVOS = holoEventMapper.selectCrossOptMonitoringList();
        List<CrossLastOptResultDTO> optResultDTOS = strategyCrossResultMapper.selectLastOneCrossOptList();
        long current = System.currentTimeMillis();
        optMonitoringVOS.forEach(optMonitoringVO -> {
            String crossId = optMonitoringVO.getId();
            for (CrossLastOptResultDTO optResultDTO : optResultDTOS) {
                String id = optResultDTO.getId();
                String type = optMonitoringVO.getType();
                optMonitoringVO.setTypeDesc(EventInfoTypeEnum.getOptDesc(type));
                if (StringUtils.endsWithIgnoreCase(id, crossId)) {
                    Date optTime = optResultDTO.getOptTime();
                    Integer duration = optResultDTO.getDuration();
                    // 优化时间计算
                    optMonitoringVO.setOptTime(optTime);
                    optMonitoringVO.setOptStatus(0);
                    if (Objects.nonNull(optTime)) {
                        // 空放优化中，6秒钟结束，当前时间小于开始时间+持续时间
                        if (Objects.equals(EventInfoTypeEnum.PHASE_EMPTY.getEventType(), type) && (current - optTime.getTime() > 6 * 1000)) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                            // 其他事件优化中，当前时间小于优化时间+持续时间
                        } else if (current - optTime.getTime() <= duration * 1000) {
                            optMonitoringVO.setOptStatus(1);
                            optMonitoringVO.setOptStatusDesc("优化中");
                        }
                    }
                }
            }
        });
        crossListCache = optMonitoringVOS;
        return optMonitoringVOS;
    }

    @Override
    public List<OptMonitoringVO> eventAlarmRealTimeList() throws Exception {
        List<OptMonitoringVO> results = new ArrayList<>(90);
        List<OptMonitoringVO> greenList = new ArrayList<>(10);
        List<OptMonitoringVO> crossList = new ArrayList<>(80);
        if (!CollectionUtils.isEmpty(greenListCache)) {
            greenList = greenListCache;
        } else {
            greenList = greenOptMonitoringList();
        }
        if (!CollectionUtils.isEmpty(crossListCache)) {
            crossList = crossListCache;
        } else {
            crossList = crossOptMonitoringList();
        }
        results.addAll(greenList);
        results.addAll(crossList);
        List<OptMonitoringVO> sort = results.stream().sorted(Comparator.comparing(OptMonitoringVO::getStartTime).reversed()).collect(Collectors.toList());
        return sort;
    }

    @Override
    public List<StatisticsEventTypeCountTimeVO> selectGreenEventTypeCountTimeList(EventTypeCountTimeVO vo) throws Exception {
        String type = vo.getType();
        LocalDateTime current = vo.getDate();
        LocalDateTime curDayStart = DateUtil.getLastDayMidNight(current, 0);
        LocalDateTime latDayStart = DateUtil.getLastDayMidNight(current, -7);
        LocalDateTime lastDayEnd = current.plusDays(-1);
        List<StatisticsEventTypeCountTimeVO> currentList = holoEventMapper.selectGreenEventTypeCountTimeList(type, curDayStart, current);
        List<StatisticsEventTypeCountTimeVO> lastDayList = holoEventMapper.selectGreenEventTypeCountTimeList(type, latDayStart, lastDayEnd);
        setCountDurationRate(currentList, lastDayList);
        return currentList;
    }

    private static void setCountDurationRate(List<StatisticsEventTypeCountTimeVO> currentList, List<StatisticsEventTypeCountTimeVO> lastDayList) {
        currentList.forEach(result -> {
            for (StatisticsEventTypeCountTimeVO last : lastDayList) {
                String id = result.getId();
                String lastId = last.getId();
                if (StringUtils.endsWithIgnoreCase(id, lastId)) {
                    if (last.getCount() == 0 || last.getDuration() == 0) {
                        continue;
                    }
                    double countOffset = (double) result.getCount() - last.getCount();
                    double durationOffset = (double) result.getDuration() - last.getDuration();
                    double countRate = countOffset / last.getCount();
                    double durationRate =  durationOffset / last.getDuration();
                    result.setCountRate((int) (Math.round(countRate * 100) ));
                    result.setDurationRate((int) (Math.round(durationRate * 100) ));
                }

            }
        });
    }

    @Override
    public List<StatisticsEventTypeCountTimeVO> selectCrossEventTypeCountTimeList(EventTypeCountTimeVO vo) throws Exception {
        String type = vo.getType();
        LocalDateTime current = vo.getDate();
        LocalDateTime curDayStart = DateUtil.getLastDayMidNight(current, 0);
        LocalDateTime latDayStart = DateUtil.getLastDayMidNight(current, -1);
        LocalDateTime lastDayEnd = current.plusDays(-1);
        List<StatisticsEventTypeCountTimeVO> currentList = holoEventMapper.selectCrossEventTypeCountTimeList(type, curDayStart, current);
        List<StatisticsEventTypeCountTimeVO> lastDayList = holoEventMapper.selectCrossEventTypeCountTimeList(type, latDayStart, lastDayEnd);
        setCountDurationRate(currentList, lastDayList);
        return currentList;
    }

    @Override
    public PageInfo<StatisticsEventTypeCountTimeVO> selectCrossGreenHistList(int pageNo, int pageSize) throws Exception {
        PageHelper.startPage(pageNo, pageSize);
        List<StatisticsEventTypeCountTimeVO> results = holoEventMapper.selectCrossGreenHistList();
        return new PageInfo<>(results);
    }

    @Override
    public String optStrategyResultInfo(String id) throws Exception {
        StringBuilder sb = new StringBuilder();
        String result = "";
        if (net.wanji.common.utils.tool.StringUtils.isNotBlank(id)) {
            if (id.length() > 1) {
                result = getOptCrossResult(id, sb);
            } else {
                result = getOptGreenResult(id, sb);
            }
        }
        return result;
    }

    private String getOptGreenResult(String id, StringBuilder sb) {
        String result = "";
        List<GreenLastOptResultDTO> greenLastOptResultDTOS = strategyGreenOptHistMapper.selectLastGreenOptResultList();
        if (!CollectionUtils.isEmpty(greenLastOptResultDTOS)) {
            for (GreenLastOptResultDTO dto : greenLastOptResultDTOS) {
                if (StringUtils.endsWithIgnoreCase(id, String.valueOf(dto.getGreenId()))) {
                    Integer controlMethod = dto.getControlMethod();
                    if (controlMethod >= 0) {
                        sb.append("动态绿波").append("-").append("均衡调控").append("-").append("神思策略");
                    } else {
                        sb.append("均衡调控").append("-").append("专家方案");
                    }
                }
            }
        } else {
            sb.append("动态绿波-专家方案");
        }
        result = sb.toString();
        return result;
    }

    private String getOptCrossResult(String id, StringBuilder sb) {
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, id);
        queryWrapper.orderByDesc(StrategyCrossResultEntity::getIssueTime);
        queryWrapper.last("limit 1");
        List<StrategyCrossResultEntity> results = strategyCrossResultMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(results)) {
            for (StrategyCrossResultEntity result : results) {
                Integer currentAlgo = result.getCurrentAlgo();
                if (Objects.equals(1, currentAlgo)) {
                    sb.append("相位空放").append("-").append("均衡调控").append("-").append("神思策略");
                }
                if (Objects.equals(2, currentAlgo)) {
                    sb.append("路口失衡").append("-").append("效率提升").append("-").append("神思策略");
                }
                if (Objects.equals(3, currentAlgo)) {
                    sb.append("路口溢出").append("-").append("均衡调控").append("-").append("神思策略");
                }
                if (Objects.equals(4, currentAlgo)) {
                    sb.append("路口拥堵").append("-").append("均衡调控").append("-").append("神思策略");
                }
            }
        }
        return sb.toString();
    }

    @Override
    public GreenOptCrossOffsetVO optStrategyCrossOffsetList(Integer id) throws Exception {
        GreenOptCrossOffsetVO greenOptCrossOffsetVO = new GreenOptCrossOffsetVO();
        try {
            List<GreenOptCrossOffsetVO.CrossOffsetDetail> oriOffsetDetails = new ArrayList<>();
            List<GreenOptCrossOffsetVO.CrossOffsetDetail> curOffsetDetails = new ArrayList<>();
            List<GreenwaveCrossPO> greenwaveCrossPOS = greenwaveCrossMapper.selectByGreenwaveId(id);
            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOS) {
                String crossId = greenwaveCrossPO.getCrossId();

                String crossName = baseCrossInfoCache.getCrossName(crossId);

                GreenOptCrossOffsetVO.CrossOffsetDetail oriOffsetDetail = getOriOffsetDetail(crossId);
                oriOffsetDetail.setCrossName(crossName);
                oriOffsetDetails.add(oriOffsetDetail);

                GreenOptCrossOffsetVO.CrossOffsetDetail curOffsetDetail = getCurOffsetDetail(crossId, oriOffsetDetail);
                curOffsetDetail.setCrossName(crossName);
                curOffsetDetails.add(curOffsetDetail);

            }
            greenOptCrossOffsetVO.setOriCrossOffsets(oriOffsetDetails);
            greenOptCrossOffsetVO.setCurCrossOffsets(curOffsetDetails);
        } catch (Exception e) {
            log.error("态势监测-策略推荐-绿波优化相位差查询失败：{}", e);
            throw new RuntimeException(e);
        }

        return greenOptCrossOffsetVO;
    }

    private GreenOptCrossOffsetVO.CrossOffsetDetail getCurOffsetDetail(String crossId, GreenOptCrossOffsetVO.CrossOffsetDetail oriDetail) throws Exception {
        ObjectMapper mapper = JacksonUtils.getInstance();
        GreenOptCrossOffsetVO.CrossOffsetDetail curDetail = new GreenOptCrossOffsetVO.CrossOffsetDetail();
        Object offsetStr = redisUtil.getHash("utc_scheme_offset", crossId);
        if (Objects.isNull(offsetStr)) {
            return oriDetail;
        } else {
            CrossSchemeRings crossSchemeRings = mapper.readValue(String.valueOf(offsetStr), CrossSchemeRings.class);
            if (Objects.nonNull(crossSchemeRings)) {
                Integer offset = StringUtils.isEmpty(crossSchemeRings.getOffset()) ? 0 : Integer.valueOf(crossSchemeRings.getOffset());
                curDetail.setOffset(offset);
                curDetail.setCrossId(crossId);
                curDetail.setSchemeNo(Integer.valueOf(crossSchemeRings.getPattern()));
            }
        }
        return curDetail;
    }

    private GreenOptCrossOffsetVO.CrossOffsetDetail getOriOffsetDetail(String crossId) throws Exception {
        ObjectMapper mapper = JacksonUtils.getInstance();
        GreenOptCrossOffsetVO.CrossOffsetDetail crossOffsetDetail = new GreenOptCrossOffsetVO.CrossOffsetDetail();
        Object lightStr = redisUtil.getHash("utc_light_status", crossId);
        if (!StringUtils.isEmpty(lightStr)) {
            List<LightsStatusVO2> lightsStatusVO2s = mapper.readValue(String.valueOf(lightStr), new TypeReference<List<LightsStatusVO2>>() {});
            String schemeId = lightsStatusVO2s.get(0).getSchemeId();
            Integer schemeNo = Integer.valueOf(schemeId);
            BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, schemeNo);
            crossOffsetDetail.setOffset(0);
            if (Objects.nonNull(baseCrossSchemePO)) {
                Integer offset = baseCrossSchemePO.getOffset();
                crossOffsetDetail.setCrossId(crossId);
                crossOffsetDetail.setSchemeNo(schemeNo);
                crossOffsetDetail.setOffset(offset);
            }
        }
        return crossOffsetDetail;
    }
}